@@ -1,6 +1,6 @@
 /* ac.c - Alternative interface for asymmetric cryptography.
    Copyright (C) 2003, 2004, 2005, 2006
-                 2007  Free Software Foundation, Inc.
+                 2007, 2008  Free Software Foundation, Inc.
  
    This file is part of Libgcrypt.
   
@@ -24,7 +24,6 @@
 #include <string.h>
 #include <stdio.h>
 #include <stddef.h>
-#include <assert.h>
 
 #include "g10lib.h"
 #include "cipher.h"
@@ -127,6 +126,9 @@ _gcry_ac_data_new (gcry_ac_data_t *data)
   gcry_ac_data_t data_new;
   gcry_error_t err;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   data_new = gcry_malloc (sizeof (*data_new));
   if (! data_new)
     {
@@ -240,6 +242,9 @@ _gcry_ac_data_copy (gcry_ac_data_t *data_cp, gcry_ac_data_t data)
   gcry_ac_data_t data_new;
   gcry_error_t err;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   /* Allocate data set.  */
   data_new = gcry_malloc (sizeof (*data_new));
   if (! data_new)
@@ -290,6 +295,9 @@ _gcry_ac_data_set (gcry_ac_data_t data, unsigned int flags,
   name_cp = NULL;
   mpi_cp = NULL;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   if (flags & ~(GCRY_AC_FLAG_DEALLOC | GCRY_AC_FLAG_COPY))
     {
       err = gcry_error (GPG_ERR_INV_ARG);
@@ -370,6 +378,9 @@ _gcry_ac_data_get_name (gcry_ac_data_t data, unsigned int flags,
   gcry_error_t err;
   unsigned int i;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   if (flags & ~(GCRY_AC_FLAG_COPY))
     {
       err = gcry_error (GPG_ERR_INV_ARG);
@@ -421,6 +432,9 @@ _gcry_ac_data_get_index (gcry_ac_data_t data, unsigned int flags,
   name_cp = NULL;
   mpi_cp = NULL;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   if (flags & ~(GCRY_AC_FLAG_COPY))
     {
       err = gcry_error (GPG_ERR_INV_ARG);
@@ -496,6 +510,9 @@ _gcry_ac_data_to_sexp (gcry_ac_data_t data, gcry_sexp_t *sexp,
   arg_list = NULL;
   err = 0;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   /* Calculate size of S-expression representation.  */
 
   i = 0;
@@ -626,6 +643,9 @@ _gcry_ac_data_from_sexp (gcry_ac_data_t *data_set, gcry_sexp_t sexp,
   mpi = NULL;
   err = 0;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   /* Process S-expression/identifiers.  */
 
   if (identifiers)
@@ -795,6 +815,9 @@ _gcry_ac_data_dump (const char *prefix, gcry_ac_data_t data)
   if (! data)
     return;
 
+  if (fips_mode ())
+    return;
+
   mpi_buffer = NULL;
 
   data_n = _gcry_ac_data_length (data);
@@ -859,8 +882,11 @@ _gcry_ac_io_init_va (gcry_ac_io_t *ac_io,
 {
   memset (ac_io, 0, sizeof (*ac_io));
 
-  assert ((mode == GCRY_AC_IO_READABLE) || (mode == GCRY_AC_IO_WRITABLE));
-  assert ((type == GCRY_AC_IO_STRING) || (type == GCRY_AC_IO_STRING));
+  if (fips_mode ())
+    return;
+
+  gcry_assert ((mode == GCRY_AC_IO_READABLE) || (mode == GCRY_AC_IO_WRITABLE));
+  gcry_assert ((type == GCRY_AC_IO_STRING) || (type == GCRY_AC_IO_STRING));
 
   ac_io->mode = mode;
   ac_io->type = type;
@@ -920,7 +946,7 @@ _gcry_ac_io_write (gcry_ac_io_t *ac_io, unsigned char *buffer, size_t buffer_n)
 {
   gcry_error_t err;
 
-  assert (ac_io->mode == GCRY_AC_IO_WRITABLE);
+  gcry_assert (ac_io->mode == GCRY_AC_IO_WRITABLE);
   err = 0;
 
   switch (ac_io->type)
@@ -980,7 +1006,7 @@ _gcry_ac_io_read (gcry_ac_io_t *ac_io,
 {
   gcry_error_t err;
   
-  assert (ac_io->mode == GCRY_AC_IO_READABLE);
+  gcry_assert (ac_io->mode == GCRY_AC_IO_READABLE);
   err = 0;
 
   switch (ac_io->type)
@@ -1362,6 +1388,9 @@ _gcry_ac_open (gcry_ac_handle_t *handle,
   *handle = NULL;
   module = NULL;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   /* Get name.  */
   algorithm_name = _gcry_pk_aliased_algo_name (algorithm);
   if (! algorithm_name)
@@ -1431,6 +1460,9 @@ _gcry_ac_key_init (gcry_ac_key_t *key, gcry_ac_handle_t handle,
 
   (void)handle;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   /* Allocate.  */
   key_new = gcry_malloc (sizeof (*key_new));
   if (! key_new)
@@ -1488,6 +1520,9 @@ _gcry_ac_key_pair_generate (gcry_ac_handle_t handle, unsigned int nbits,
 
   (void)misc_data;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   key_data_secret = NULL;
   key_data_public = NULL;
   key_secret = NULL;
@@ -1496,6 +1531,7 @@ _gcry_ac_key_pair_generate (gcry_ac_handle_t handle, unsigned int nbits,
   arg_list = NULL;
   genkey_sexp_request = NULL;
   genkey_sexp_reply = NULL;
+  key_sexp = NULL;
 
   /* Allocate key pair.  */
   key_pair_new = gcry_malloc (sizeof (struct gcry_ac_key_pair));
@@ -1631,6 +1667,7 @@ _gcry_ac_key_pair_generate (gcry_ac_handle_t handle, unsigned int nbits,
   gcry_free (arg_list);
   gcry_sexp_release (genkey_sexp_request);
   gcry_sexp_release (genkey_sexp_reply);
+  gcry_sexp_release (key_sexp);
   if (err)
     {
       _gcry_ac_data_destroy (key_data_secret);
@@ -1650,6 +1687,9 @@ _gcry_ac_key_pair_extract (gcry_ac_key_pair_t key_pair,
 {
   gcry_ac_key_t key;
 
+  if (fips_mode ())
+    return NULL;
+
   switch (which)
     {
     case GCRY_AC_KEY_SECRET:
@@ -1679,8 +1719,13 @@ _gcry_ac_key_destroy (gcry_ac_key_t key)
       if (key->data)
         {
           for (i = 0; i < key->data->data_n; i++)
-            if (key->data->data[i].mpi != NULL)
-              gcry_mpi_release (key->data->data[i].mpi);
+            {
+              if (key->data->data[i].mpi)
+                gcry_mpi_release (key->data->data[i].mpi);
+              if (key->data->data[i].name)
+                gcry_free (key->data->data[i].name);
+            }
+          gcry_free (key->data->data);
           gcry_free (key->data);
         }
       gcry_free (key);
@@ -1703,6 +1748,8 @@ _gcry_ac_key_pair_destroy (gcry_ac_key_pair_t key_pair)
 gcry_ac_data_t
 _gcry_ac_key_data_get (gcry_ac_key_t key)
 {
+  if (fips_mode ())
+    return NULL;
   return key->data;
 }
 
@@ -1713,6 +1760,9 @@ _gcry_ac_key_test (gcry_ac_handle_t handle, gcry_ac_key_t key)
   gcry_sexp_t key_sexp;
   gcry_error_t err;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   key_sexp = NULL;
   err = ac_data_construct (ac_key_identifiers[key->type], 0, 0,
 			   handle->algorithm_name, key->data, &key_sexp);
@@ -1737,6 +1787,9 @@ _gcry_ac_key_get_nbits (gcry_ac_handle_t handle,
   gcry_error_t err;
   unsigned int n;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   key_sexp = NULL;
 
   err = ac_data_construct (ac_key_identifiers[key->type],
@@ -1770,6 +1823,9 @@ _gcry_ac_key_get_grip (gcry_ac_handle_t handle,
   gcry_error_t err;
   unsigned char *ret;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   key_sexp = NULL;
   err = ac_data_construct (ac_key_identifiers[key->type], 0, 0,
 			   handle->algorithm_name, key->data, &key_sexp);
@@ -1816,6 +1872,9 @@ _gcry_ac_data_encrypt (gcry_ac_handle_t handle,
   gcry_sexp_t sexp_key;
   gcry_error_t err;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   data_encrypted_new = NULL;
   sexp_request = NULL;
   sexp_reply = NULL;
@@ -1890,6 +1949,9 @@ _gcry_ac_data_decrypt (gcry_ac_handle_t handle,
   gcry_sexp_t sexp_key;
   gcry_error_t err;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   sexp_request = NULL;
   sexp_reply = NULL;
   sexp_value = NULL;
@@ -1962,6 +2024,9 @@ _gcry_ac_data_sign (gcry_ac_handle_t handle,
   gcry_sexp_t sexp_key;
   gcry_error_t err;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   data_signed = NULL;
   data_value = NULL;
   sexp_request = NULL;
@@ -2032,6 +2097,9 @@ _gcry_ac_data_verify (gcry_ac_handle_t handle,
   gcry_sexp_t sexp_key;
   gcry_error_t err;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   sexp_signature = NULL;
   data_value = NULL;
   sexp_data = NULL;
@@ -2502,6 +2570,9 @@ _gcry_ac_data_encode (gcry_ac_em_t method,
 		      gcry_ac_io_t *ac_io_read,
 		      gcry_ac_io_t *ac_io_write)
 {
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   return ac_data_dencode (method, DATA_ENCODE, flags, options,
 			  ac_io_read, ac_io_write);
 }
@@ -2515,6 +2586,9 @@ _gcry_ac_data_decode (gcry_ac_em_t method,
 		      gcry_ac_io_t *ac_io_read,
 		      gcry_ac_io_t *ac_io_write)
 {
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   return ac_data_dencode (method, DATA_DECODE, flags, options,
 			  ac_io_read, ac_io_write);
 }
@@ -2530,6 +2604,9 @@ _gcry_ac_mpi_to_os (gcry_mpi_t mpi, unsigned char *os, size_t os_n)
   gcry_mpi_t m;
   gcry_mpi_t d;
 
+  if (fips_mode ())
+    return;
+
   base = gcry_mpi_new (0);
   gcry_mpi_set_ui (base, 256);
 
@@ -2568,6 +2645,9 @@ _gcry_ac_mpi_to_os_alloc (gcry_mpi_t mpi, unsigned char **os, size_t *os_n)
   gcry_error_t err;
   unsigned int nbits;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   nbits = gcry_mpi_get_nbits (mpi);
   buffer_n = (nbits + 7) / 8;
   buffer = gcry_malloc (buffer_n);
@@ -2597,6 +2677,9 @@ _gcry_ac_os_to_mpi (gcry_mpi_t mpi, unsigned char *os, size_t os_n)
   gcry_mpi_t x;
   gcry_mpi_t a;
   
+  if (fips_mode ())
+    return;
+
   a = gcry_mpi_new (0);
   gcry_mpi_set_ui (a, 1);
   x = gcry_mpi_new (0);
@@ -2815,6 +2898,9 @@ _gcry_ac_data_encrypt_scheme (gcry_ac_handle_t handle,
 
   (void)flags;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   data_encrypted = NULL;
   mpi_encrypted = NULL;
   mpi_plain = NULL;
@@ -2906,6 +2992,9 @@ _gcry_ac_data_decrypt_scheme (gcry_ac_handle_t handle,
 
   (void)flags;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   data_encrypted = NULL;
   mpi_encrypted = NULL;
   mpi_decrypted = NULL;
@@ -3017,6 +3106,9 @@ _gcry_ac_data_sign_scheme (gcry_ac_handle_t handle,
 
   (void)flags;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   data_signed = NULL;
   mpi_signed = NULL;
   opts_em = NULL;
@@ -3109,6 +3201,9 @@ _gcry_ac_data_verify_scheme (gcry_ac_handle_t handle,
 
   (void)flags;
 
+  if (fips_mode ())
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   mpi_signature = NULL;
   elements_sig = NULL;
   data_signed = NULL;
@@ -3199,5 +3294,8 @@ _gcry_ac_data_verify_scheme (gcry_ac_handle_t handle,
 gcry_err_code_t
 _gcry_ac_init (void)
 {
+  if (fips_mode ())
+    return GPG_ERR_NOT_SUPPORTED;
+
   return 0;
 }
