@@ -1316,7 +1316,8 @@ check_digests (void)
 
   for (i = 0; algos[i].md; i++)
     {
-      if (gcry_md_test_algo (algos[i].md) && in_fips_mode)
+      if ((gcry_md_test_algo (algos[i].md) || algos[i].md == GCRY_MD_MD5)
+          && in_fips_mode)
         {
           if (verbose)
             fprintf (stderr, "  algorithm %d not available in fips mode\n",
@@ -1685,7 +1686,8 @@ check_hmac (void)
 
   for (i = 0; algos[i].md; i++)
     {
-      if (gcry_md_test_algo (algos[i].md) && in_fips_mode)
+      if ((gcry_md_test_algo (algos[i].md) || algos[i].md == GCRY_MD_MD5)
+          && in_fips_mode)
         {
           if (verbose)
             fprintf (stderr, "  algorithm %d not available in fips mode\n",
@@ -1693,10 +1695,11 @@ check_hmac (void)
           continue;
         }
       if (verbose)
-	fprintf (stderr, "  checking %s [%i] for length %zi\n", 
+	fprintf (stderr, 
+                 "  checking %s [%i] for %zi byte key and %zi byte data\n", 
 		 gcry_md_algo_name (algos[i].md),
 		 algos[i].md,
-		 strlen(algos[i].data));
+		 strlen(algos[i].key), strlen(algos[i].data));
 
       check_one_hmac (algos[i].md, algos[i].data, strlen (algos[i].data),
 		      algos[i].key, strlen(algos[i].key),
@@ -2117,6 +2120,7 @@ main (int argc, char **argv)
       check_pubkey ();
     }
 
+
   if (in_fips_mode && !selftest_only)
     {
       /* If we are in fips mode do some more tests. */
@@ -2164,11 +2168,15 @@ main (int argc, char **argv)
   else
     {
       /* If in standard mode, run selftests.  */
-      gcry_control (GCRYCTL_SELFTEST, 0);
+      if (gcry_control (GCRYCTL_SELFTEST, 0))
+        fail ("running self-test failed\n");
     }
 
   if (verbose)
     fprintf (stderr, "\nAll tests completed. Errors: %i\n", error_count);
 
+  if (in_fips_mode && !gcry_fips_mode_active ())
+    fprintf (stderr, "FIPS mode is not anymore active\n");
+
   return error_count ? 1 : 0;
 }
