@@ -29,7 +29,13 @@
 #else
 #include <sys/times.h>
 #endif
-#include <gcrypt.h>
+
+#ifdef _GCRYPT_IN_LIBGCRYPT
+# include "../src/gcrypt.h"
+#else
+# include <gcrypt.h>
+#endif
+
 
 #define PGM "benchmark"
 
@@ -41,6 +47,9 @@ static int large_buffers;
 /* Number of cipher repetitions.  */
 static int cipher_repetitions;
 
+/* Whether fips mode was active at startup.  */
+static int in_fips_mode;
+
 
 static const char sample_private_dsa_key_1024[] =
 "(private-key\n"
@@ -373,7 +382,9 @@ md_bench ( const char *algoname )
   if (!algoname)
     {
       for (i=1; i < 400; i++)
-        if ( !gcry_md_test_algo (i) )
+        if (in_fips_mode && i == GCRY_MD_MD5)
+          ; /* Don't use MD5 in fips mode.  */
+        else if ( !gcry_md_test_algo (i) )
           md_bench (gcry_md_algo_name (i));
       return;
     }
@@ -629,7 +640,7 @@ rsa_bench (int iterations, int print_header, int no_blinding)
       fflush (stdout);
 
       err = gcry_sexp_build (&key_spec, NULL,
-                             gcry_control (GCRYCTL_FIPS_MODE_P, 0)
+                             gcry_fips_mode_active ()
                              ? "(genkey (RSA (nbits %d)))"
                              : "(genkey (RSA (nbits %d)(transient-key)))",
                              p_sizes[testno]);
@@ -1054,7 +1065,11 @@ main( int argc, char **argv )
       fprintf (stderr, PGM ": version mismatch\n");
       exit (1);
     }
-  gcry_control (GCRYCTL_DISABLE_SECMEM, 0);
+
+  if (gcry_fips_mode_active ())
+    in_fips_mode = 1;
+  else
+    gcry_control (GCRYCTL_DISABLE_SECMEM, 0);
 
   if (use_random_daemon)
     gcry_control (GCRYCTL_USE_RANDOM_DAEMON, 1);
@@ -1133,6 +1148,10 @@ main( int argc, char **argv )
       fprintf (stderr, PGM ": bad arguments\n");
       return 1;
     }
+
+
+  if (in_fips_mode && !gcry_fips_mode_active ())
+    fprintf (stderr, PGM ": FIPS mode is not anymore active\n");
   
   return 0;
 }
